package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 投诉内容管理Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "投诉内容管理Dto对象")
@Data
public class TourImComplaitDto {

    /**
     * id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 举报类型id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "举报类型id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，举报类型id不能为空！")
    private Long typeId;

    /**
     * 类型为其他违规的理由。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "类型为其他违规的理由。可支持等于操作符的列表数据过滤。")
    private String elseTypeReason;

    /**
     * 举报对象 1用户 2群组。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "举报对象 1用户 2群组。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，举报对象 1用户 2群组不能为空！")
    private Integer objectType;

    /**
     * 举报描述。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "举报描述。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，举报描述不能为空！")
    private String description;

    /**
     * 举报图片 最多三张。
     */
    @Schema(description = "举报图片 最多三张。")
    private String image;

    /**
     * 用户id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "用户id。可支持等于操作符的列表数据过滤。")
    private Long userId;

    /**
     * 群组id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "群组id。可支持等于操作符的列表数据过滤。")
    private Long groupId;

    /**
     * else_type_reason / description LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
