package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;
import com.tourism.common.core.validator.ConstDictRef;
import com.tourism.webadmin.back.model.constant.IsTrue;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

/**
 * 聊天群聊消息Dto对象。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Schema(description = "聊天群聊消息Dto对象")
@Data
public class TourImMessageDto {

    /**
     * id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 消息内容。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "消息内容。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，消息内容不能为空！")
    private String messageContent;

    /**
     * 消息类型 0文字 1文件 2系统消息。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "消息类型 0文字 1文件 2系统消息。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，消息类型 0文字 1文件 2系统消息不能为空！", groups = {UpdateGroup.class})
    private Integer messageType;

    /**
     * 聊天群组id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "聊天群组id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，聊天群组id不能为空！", groups = {UpdateGroup.class})
    private Long groupId;

    /**
     * 通知消息类型 1.单聊消息 2 群聊消息 3系统消息 4关注信息 5点赞 6评论 7评论区艾特 8文章内艾特 9访问。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "通知消息类型 1.单聊消息 2 群聊消息 3系统消息 4关注信息 5点赞 6评论 7评论区艾特 8文章内艾特 9访问。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，通知消息类型 1.单聊消息 2 群聊消息 3系统消息 4关注信息 5点赞 6评论 7评论区艾特 8文章内艾特 9访问不能为空！")
    private Integer noticeType;

    /**
     * 消息置顶/公告 0不置顶 1置顶。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "消息置顶/公告 0不置顶 1置顶。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，消息置顶/公告 0不置顶 1置顶不能为空！", groups = {UpdateGroup.class})
    @ConstDictRef(constDictClass = IsTrue.class, message = "数据验证失败，消息置顶/公告 0不置顶 1置顶为无效值！")
    private Integer isTop;

    /**
     * 撤回标记 0不撤回 1撤回。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "撤回标记 0不撤回 1撤回。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，撤回标记 0不撤回 1撤回不能为空！", groups = {UpdateGroup.class})
    @ConstDictRef(constDictClass = IsTrue.class, message = "数据验证失败，撤回标记 0不撤回 1撤回为无效值！")
    private Integer revocationTag;

    /**
     * 已经删除该条消息的用户。
     */
    @Schema(description = "已经删除该条消息的用户。")
    private String deletedByUsers;

    /**
     * 已读该条消息的用户。
     */
    @Schema(description = "已读该条消息的用户。")
    private String readByUsers;

    /**
     * message_content LIKE搜索字符串。
     * NOTE: 可支持LIKE操作符的列表数据过滤。
     */
    @Schema(description = "LIKE模糊搜索字符串")
    private String searchString;
}
