package com.tourism.webadmin.back.dto;

import com.tourism.common.core.validator.UpdateGroup;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import jakarta.validation.constraints.*;

import java.util.Date;

/**
 * 游记列表搜索Dto对象。
 *
 * @author 奇幻之旅
 * @date 2024-09-06
 */
@Schema(description = "游记列表搜索Dto对象")
@Data
public class TourTourismSearchRecordsDto {

    /**
     * 主键id。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "主键id。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，主键id不能为空！", groups = {UpdateGroup.class})
    private Long id;

    /**
     * 搜索关键词。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "搜索关键词。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotBlank(message = "数据验证失败，搜索关键词不能为空！")
    private String searchKeyword;

    /**
     * 搜索结果数量。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "搜索结果数量。可支持等于操作符的列表数据过滤。")
    private Integer searchResultCount;

    /**
     * 搜索时间。
     * NOTE: 可支持等于操作符的列表数据过滤。
     */
    @Schema(description = "搜索时间。可支持等于操作符的列表数据过滤。", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "数据验证失败，搜索时间不能为空！")
    private Date searchTime;

    /**
     * searchResultCount 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "searchResultCount 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private Integer searchResultCountStart;

    /**
     * searchResultCount 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "searchResultCount 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private Integer searchResultCountEnd;

    /**
     * searchTime 范围过滤起始值(>=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "searchTime 范围过滤起始值(>=)。可支持范围操作符的列表数据过滤。")
    private String searchTimeStart;

    /**
     * searchTime 范围过滤结束值(<=)。
     * NOTE: 可支持范围操作符的列表数据过滤。
     */
    @Schema(description = "searchTime 范围过滤结束值(<=)。可支持范围操作符的列表数据过滤。")
    private String searchTimeEnd;
}
