package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.common.core.object.TokenData;
import com.tourism.webadmin.app.website.model.TourShortUser;
import com.tourism.webadmin.app.website.vo.TourShortUserVo;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.codehaus.groovy.syntax.TokenUtil;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 关注粉丝管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourFansService")
public class TourFansServiceImpl extends BaseService<TourFans, Long> implements TourFansService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourFansMapper tourFansMapper;
    @Autowired
    private TourUserService tourUserService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourFans> mapper() {
        return tourFansMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourFans saveNew(TourFans tourFans) {
        tourFansMapper.insert(this.buildDefaultValue(tourFans));
        return tourFans;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourFans> tourFansList) {
        if (CollUtil.isNotEmpty(tourFansList)) {
            tourFansList.forEach(this::buildDefaultValue);
            tourFansMapper.insertList(tourFansList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourFans> tourFansList, int batchSize) {
        if (CollUtil.isEmpty(tourFansList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourFansList.size(), start + batchSize);
            List<TourFans> subList = tourFansList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourFansMapper.insertList(subList);
            if (end == tourFansList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourFans tourFans, TourFans originalTourFans) {
        tourFans.setUpdateUserId(TokenData.takeFromRequest().getUserId());
        tourFans.setUpdateTime(new Date());
//        MyModelUtil.fillCommonsForUpdate(tourFans, originalTourFans);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        return tourFansMapper.updateById(tourFans)==1 ;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourFansMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourFans> getTourFansList(TourFans filter, String orderBy) {
        return tourFansMapper.getTourFansList(filter, orderBy);
    }

    @Override
    public List<TourFans> getTourFansListWithRelation(TourFans filter, String orderBy) {
        List<TourFans> resultList = tourFansMapper.getTourFansList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourFans> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "attentionId")) {
            callResult = verifyImportForDatasourceDict(dataList, "attentionIdDictMap", TourFans::getAttentionId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "createUserId")) {
            callResult = verifyImportForDatasourceDict(dataList, "createUserIdDictMap", TourFans::getCreateUserId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(TourFans tourFans, TourFans originalTourFans) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourFans, originalTourFans, TourFans::getAttentionId)
                && !tourUserService.existId(tourFans.getAttentionId())) {
            return CallResult.error(String.format(errorMessageFormat, "被关注人id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourFans, originalTourFans, TourFans::getCreateUserId)
                && !tourUserService.existId(tourFans.getCreateUserId())) {
            return CallResult.error(String.format(errorMessageFormat, "创建人id"));
        }
        return CallResult.ok();
    }

    private TourFans buildDefaultValue(TourFans tourFans) {
        if (tourFans.getId() == null) {
            tourFans.setId(idGenerator.nextLongId());
        }
        tourFans.setCreateTime(new Date());
        tourFans.setUpdateTime(new Date());
        tourFans.setUpdateUserId(TokenData.takeFromRequest().getUserId());
//        MyModelUtil.fillCommonsForInsert(tourFans);
        tourFans.setDeleteFlag(GlobalDeletedFlag.NORMAL);
        return tourFans;
    }

    @Override
    public List<TourShortUser> getFouceEachFriendsByName(Long userId, String name) {
        return tourFansMapper.getFouceEachFriendsByName(userId,name);
    }
}
