package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.github.pagehelper.page.PageMethod;
import com.github.pagehelper.page.PageParams;
import com.tourism.common.core.constant.ErrorCodeEnum;
import com.tourism.common.core.object.MyPageData;
import com.tourism.common.core.object.ResponseResult;
import com.tourism.webadmin.app.website.dto.DelMessageDto;
import com.tourism.webadmin.app.website.vo.GroupMessageVo;
import com.tourism.webadmin.back.model.constant.IsTrue;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.vo.TourImMessageVo;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import com.volcengine.model.video_aiot.response.GetRecordResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.yaml.snakeyaml.Yaml;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 聊天群聊消息数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourImMessageService")
public class TourImMessageServiceImpl extends BaseService<TourImMessage, Long> implements TourImMessageService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourImMessageMapper tourImMessageMapper;
    @Autowired
    private TourImGroupService tourImGroupService;
    @Autowired
    private GlobalDictService globalDictService;
    @Autowired
    private TourUserService tourUserService;


    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourImMessage> mapper() {
        return tourImMessageMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourImMessage saveNew(TourImMessage tourImMessage) {
        tourImMessageMapper.insert(this.buildDefaultValue(tourImMessage));
        return tourImMessage;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourImMessage> tourImMessageList) {
        if (CollUtil.isNotEmpty(tourImMessageList)) {
            tourImMessageList.forEach(this::buildDefaultValue);
            tourImMessageMapper.insertList(tourImMessageList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourImMessage> tourImMessageList, int batchSize) {
        if (CollUtil.isEmpty(tourImMessageList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourImMessageList.size(), start + batchSize);
            List<TourImMessage> subList = tourImMessageList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourImMessageMapper.insertList(subList);
            if (end == tourImMessageList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourImMessage tourImMessage, TourImMessage originalTourImMessage) {
        MyModelUtil.fillCommonsForUpdate(tourImMessage, originalTourImMessage);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourImMessage> uw = this.createUpdateQueryForNullValue(tourImMessage, tourImMessage.getId());
        return tourImMessageMapper.update(tourImMessage, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourImMessageMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourImMessage> getTourImMessageList(TourImMessage filter, String orderBy) {
        return tourImMessageMapper.getTourImMessageList(filter, orderBy);
    }

    @Override
    public List<TourImMessage> getTourImMessageListWithRelation(TourImMessage filter, String orderBy) {
        List<TourImMessage> resultList = tourImMessageMapper.getTourImMessageList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourImMessage> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "isTop")) {
            callResult = verifyImportForConstDict(dataList, "isTopDictMap", TourImMessage::getIsTop);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "revocationTag")) {
            callResult = verifyImportForConstDict(dataList, "revocationTagDictMap", TourImMessage::getRevocationTag);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "messageType")) {
            callResult = verifyImportForGlobalDict(dataList, "messageTypeDictMap", TourImMessage::getMessageType);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "noticeType")) {
            callResult = verifyImportForGlobalDict(dataList, "noticeTypeDictMap", TourImMessage::getNoticeType);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "groupId")) {
            callResult = verifyImportForDatasourceDict(dataList, "groupIdDictMap", TourImMessage::getGroupId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }
    @Override
    public CallResult verifyRelatedData(TourImMessage tourImMessage, TourImMessage originalTourImMessage) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourImMessage, originalTourImMessage, TourImMessage::getMessageType)
                && !globalDictService.existDictItemFromCache("NoticeType", tourImMessage.getMessageType())) {
            return CallResult.error(String.format(errorMessageFormat, "消息类型 0文字 1文件 2系统消息"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourImMessage, originalTourImMessage, TourImMessage::getGroupId)
                && !tourImGroupService.existId(tourImMessage.getGroupId())) {
            return CallResult.error(String.format(errorMessageFormat, "聊天群组id"));
        }
        //这里是基于字典的验证。
        if (this.needToVerify(tourImMessage, originalTourImMessage, TourImMessage::getNoticeType)
                && !globalDictService.existDictItemFromCache("TourImNoticeType", tourImMessage.getNoticeType())) {
            return CallResult.error(String.format(errorMessageFormat, "通知消息类型 1.单聊消息 2 群聊消息 3系统消息 4关注信息 5点赞 6评论 7评论区艾特 8文章内艾特 9访问"));
        }
        return CallResult.ok();
    }

    private TourImMessage buildDefaultValue(TourImMessage tourImMessage) {
        if (tourImMessage.getId() == null) {
            tourImMessage.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourImMessage);
        tourImMessage.setDataState(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(tourImMessage, "messageType", 0);
        MyModelUtil.setDefaultValue(tourImMessage, "isTop", IsTrue.FALSE);
        MyModelUtil.setDefaultValue(tourImMessage, "revocationTag", IsTrue.FALSE);
        return tourImMessage;
    }
}
